/**************************************************************************//**
*   @file   ad8403.c
*   @brief  Implementation of AD8403 Driver for NIOS processor.
*   @author DBogdan (dragos.bogdan@analog.com)
*
*******************************************************************************
* Copyright 2011(c) Analog Devices, Inc.
*
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*  - Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in
*    the documentation and/or other materials provided with the
*    distribution.
*  - Neither the name of Analog Devices, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*  - The use of this software may or may not infringe the patent rights
*    of one or more patent holders.  This license does not release you
*    from the requirement that you obtain separate licenses from these
*    patent holders to use this software.
*  - Use of the software either in source or binary form, must be run
*    on or directly connected to an Analog Devices Inc. component.
*
* THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*******************************************************************************
*   SVN Revision: 158
******************************************************************************/
#define AD8403_GLOBALS

/*****************************************************************************/
/* Include Files															 */
/*****************************************************************************/
#include "ad8403.h"
#include <altera_avalon_pio_regs.h>	//PIO access macros
#include <altera_avalon_spi.h>		//SPI functions
#include <unistd.h>					//usleep();
#include <system.h>
#if (USE_AD8403 == 1)

/*****************************************************************************/
/* Private variables                                                         */
/*****************************************************************************/
static unsigned int 	serialWord		= 0;	// The 10 bits word which will be sent to the AD8403 circuit.
unsigned char			txBuffer[4] 	= {0,};	// Tx SPI Buffer.
unsigned char   		rxBuffer[4] 	= {0,};	// Rx SPI Buffer.

/****************************************************************************/
/*  Aliases for the uC Probe data                                           */
/****************************************************************************/
volatile unsigned char* resetButton		= &AD8403_data[0].uchData;	// Position of the Reset button.
volatile unsigned char* shdnButton		= &AD8403_data[1].uchData;	// Position of the SHDN button.
volatile unsigned char* a1				= &AD8403_data[2].uchData;	// Address bit A1 (MSB)
volatile unsigned char* a0				= &AD8403_data[3].uchData;	// Address bit A0 (MSB)
volatile unsigned char* d7				= &AD8403_data[4].uchData;	// Data bit D7 (MSB)
volatile unsigned char* d6				= &AD8403_data[5].uchData;	// Data bit D6
volatile unsigned char* d5				= &AD8403_data[6].uchData;	// Data bit D5
volatile unsigned char* d4				= &AD8403_data[7].uchData;	// Data bit D4
volatile unsigned char* d3				= &AD8403_data[8].uchData;	// Data bit D3
volatile unsigned char* d2				= &AD8403_data[9].uchData;	// Data bit D2
volatile unsigned char* d1				= &AD8403_data[10].uchData;	// Data bit D1
volatile unsigned char* d0				= &AD8403_data[11].uchData;	// Data bit D0 (LSB)
volatile unsigned char* sendButton		= &AD8403_data[12].uchData;	// Position of the Send Data button.
volatile unsigned char* loadRdac1Button	= &AD8403_data[13].uchData;	// Position of the Load RDAC1 button.
volatile unsigned char* loadRdac2Button	= &AD8403_data[14].uchData;	// Position of the Load RDAC2 button.
volatile unsigned char* loadRdac3Button	= &AD8403_data[15].uchData;	// Position of the Load RDAC3 button.
volatile unsigned char* loadRdac4Button	= &AD8403_data[16].uchData;	// Position of the Load RDAC4 button.
volatile unsigned char* rdac1Value		= &AD8403_data[17].uchData;	// Value for the RDAC1 register.
volatile unsigned char* rdac2Value		= &AD8403_data[18].uchData;	// Value for the RDAC1 register.
volatile unsigned char* rdac3Value		= &AD8403_data[19].uchData;	// Value for the RDAC1 register.
volatile unsigned char* rdac4Value		= &AD8403_data[20].uchData;	// Value for the RDAC1 register.

/**************************************************************************//**
 * @brief Initialize SPI and Initial Values for AD8403 Board.
 *
 * @param None.
 *
 * @return SUCCESS.
******************************************************************************/
int AD8403_Init(void)
{
	IOWR_ALTERA_AVALON_PIO_DATA(GPIO_BASE, 0);					// Reset data.
	IOWR_ALTERA_AVALON_PIO_DIRECTION(GPIO_BASE, GPIO_SHDN
											  | GPIO_RESET
											  | GPIO_MUXA0
											  | GPIO_MUXA1
											  | GPIO_MUXA2);	// Output.
	IOWR_ALTERA_AVALON_PIO_SET_BITS(GPIO_BASE, GPIO_SHDN		// Set pins GPIO_SHDN, GPIO_RESET.
											 | GPIO_RESET);
	IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(GPIO_BASE, GPIO_MUXA0		// Clear pins GPIO_MUXA0, GPIO_MUXA1, GPIO_MUXA2.
											   | GPIO_MUXA1
											   | GPIO_MUXA2);
	SetCtrl(AD8403_SPI_CFG);

	return(SUCCESS);
}

/**************************************************************************//**
 * @brief Update the device's monitored and controlled parameters
 *
 * @param None
 *
 * @return Always returns success
******************************************************************************/
int AD8403_UpdateParameters(void)
{
	if(*resetButton)	// If the Reset button is pressed, sets GPIO_RESET pin.
	{
		IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(GPIO_BASE, GPIO_RESET);
	}
	else				// Else, clears GPIO_RESET pin.
	{
		IOWR_ALTERA_AVALON_PIO_SET_BITS(GPIO_BASE, GPIO_RESET);
	}
	if(*shdnButton)		// If the SHDN button is pressed, sets GPIO_SHDN pin.
	{
		IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(GPIO_BASE, GPIO_SHDN);
	}
	else				// Else, clears GPIO_SHDN pin.
	{
		IOWR_ALTERA_AVALON_PIO_SET_BITS(GPIO_BASE, GPIO_SHDN);
	}
	if(*sendButton)		// If the Send Data button was pressed, sends the value from the switches to the SPI.
	{
		serialWord = (*a1 << 9) + (*a0 << 8) + (*d7 << 7) + (*d6 << 6) + (*d5 << 5) +
					   (*d4 << 4) + (*d3 << 3) + (*d2 << 2) + (*d1 << 1) + *d0;
		AD8403_TxSpi(serialWord);
		*sendButton = 0;
	}
	if(*loadRdac1Button)// If the Load RDAC1 button was pressed, loads the value from the Slider 1 to the RDAC1 reg.
	{
		AD8403_TxSpi(RDAC1_ADDRESS + *rdac1Value);
		*loadRdac1Button = 0;
	}
	if(*loadRdac2Button)// If the Load RDAC2 button was pressed, loads the value from the Slider 2 to the RDAC2 reg.
	{
		AD8403_TxSpi(RDAC2_ADDRESS + *rdac2Value);
		*loadRdac2Button = 0;
	}
	if(*loadRdac3Button)// If the Load RDAC3 button was pressed, loads the value from the Slider 3 to the RDAC3 reg.
	{
		AD8403_TxSpi(RDAC3_ADDRESS + *rdac3Value);
		*loadRdac3Button = 0;
	}
	if(*loadRdac4Button)// If the Load RDAC4 button was pressed, loads the value from the Slider 4 to the RDAC4 reg.
	{
		AD8403_TxSpi(RDAC4_ADDRESS + *rdac4Value);
		*loadRdac4Button = 0;
	}

	return(SUCCESS);
}

/**************************************************************************//**
 * @brief Creates a delay.
 *
 * @param ms - Number of milliseconds.
 *
 * @return None.
******************************************************************************/
void AD8403_DelayMs(unsigned int ms)
{
	unsigned int counter;
	for(counter = 0; counter < ms; counter ++)
	{
		usleep(1000);
	}
}

/**************************************************************************//**
 * @brief Transmits 16 bits on SPI.
 *
 * @param value - Data which will be transmitted.
 *
 * @return None
******************************************************************************/
void AD8403_TxSpi(unsigned int value)
{
	txBuffer[0] = (unsigned char) ((value & 0x0000ff00) >> 8);
	txBuffer[1] = (unsigned char)  (value & 0x000000ff);
    alt_avalon_spi_command(AD8403_SPI,  // base address of SPI module
    					   0,           // slave number
    					   2,           // write_length
    					   txBuffer,    // *write_data
    					   0,           // read_length
    					   rxBuffer,    // *read_data
    					   0);          // flags
}
#endif /* USE_AD8403 */
